import { test } from "$TEST_HELPER"

import { LineChecker } from "$PROJECT/LineChecker.src"

function $verify(text, expectations) {
  let checker = new LineChecker(text);
  let stats = checker.getStats();

  test.is(stats[LineChecker.TYPE_NONE], expectations.none || 0);
  test.is(stats[LineChecker.TYPE_CR], expectations.cr || 0);
  test.is(stats[LineChecker.TYPE_LF], expectations.lf || 0);
  test.is(stats[LineChecker.TYPE_CRLF], expectations.crlf || 0);
}

test(function empty() {
  $verify("", { none: 0, cr: 0, lf: 0, crlf: 0 });
})

test(function single_line_CRLF() {
  let text =
    "This ends with CRLF." + "\r\n";

  $verify(text, { crlf: 1 });
})

test(function single_line_CR() {
  let text =
    "This ends with CR." + "\r";

  $verify(text, { cr: 1 });
})

test(function single_line_LF() {
  let text =
    "This ends with LF." + "\n";

  $verify(text, { lf: 1 });
})

test(function single_line_no_terminator() {
  let text =
    "This ends with no line terminator";

  $verify(text, { none: 1 });
})

test(function multi_line_CRLF_no_terminator() {
  let text =
    "first line" + "\r\n" +
    "second line";

  $verify(text, { none: 1, crlf: 1 });
})

test(function multi_line_CRLF_with_terminator() {
  let text =
    "first line" + "\r\n" +
    "second line" + "\r\n";

  $verify(text, { crlf: 2 });
})

test(function multi_line_LF_no_terminator() {
  let text =
    "first line" + "\n" +
    "second line";

  $verify(text, { none: 1, lf: 1 });
})

test(function multi_line_LF_with_terminator() {
  let text =
    "first line" + "\n" +
    "second line" + "\n";

  $verify(text, { lf: 2 });
})

test(function multi_line_CR_no_terminator() {
  let text =
    "first line" + "\r" +
    "second line";

  $verify(text, { none: 1, cr: 1 });
})

test(function multi_line_CR_with_terminator() {
  let text =
    "first line" + "\r" +
    "second line" + "\r";

  $verify(text, { cr: 2 });
})

test(function multi_line_mixed_no_terminator() {
  let text =
    "This uses inconsistent line terminator conventions" + "\r\n" +
    "Most lines use CRLF (including this one)." + "\r\n" +
    "This one uses Unix-style LF, though." + "\n" +
    "Then we switch back to CRLF." + "\r\n" +
    "But the file ends abruptly with no terminator";

  $verify(text, { none: 1, lf: 1, crlf: 3 });
})

test(function multi_line_mixed_with_terminator() {
  let text =
    "This uses inconsistent line terminator conventions" + "\r\n" +
    "Most lines use CRLF (including this one)." + "\r\n" +
    "This one uses Unix-style LF, though." + "\n" +
    "Then we switch back to CRLF." + "\r\n" +
    "And even this line ends with that sequence." + "\r\n";

  $verify(text, { lf: 1, crlf: 4 });
})
